package javax.management;

import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import com.sun.jmx.mbeanserver.Util;
import java.util.Hashtable;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.io.ObjectStreamField;

public class ObjectName implements Comparable<ObjectName>, QueryExp
{
    private static final long oldSerialVersionUID = -5467795090068647408L;
    private static final long newSerialVersionUID = 1081892073854801359L;
    private static final ObjectStreamField[] oldSerialPersistentFields;
    private static final ObjectStreamField[] newSerialPersistentFields;
    private static final long serialVersionUID;
    private static final ObjectStreamField[] serialPersistentFields;
    private static boolean compat;
    private static final ObjectName.Property[] _Empty_property_array;
    private transient String _canonicalName;
    private transient ObjectName.Property[] _kp_array;
    private transient ObjectName.Property[] _ca_array;
    private transient int _domain_length;
    private transient Map<String, String> _propertyList;
    private transient boolean _domain_pattern;
    private transient boolean _property_list_pattern;
    private transient boolean _property_value_pattern;
    public static final ObjectName WILDCARD;
    
    private void construct(String s) throws MalformedObjectNameException {
        if (s == null) {
            throw new NullPointerException("name cannot be null");
        }
        if (s.length() == 0) {
            this._canonicalName = "*:*";
            this._kp_array = ObjectName._Empty_property_array;
            this._ca_array = ObjectName._Empty_property_array;
            this._domain_length = 1;
            this._propertyList = null;
            this._domain_pattern = true;
            this._property_list_pattern = true;
            this._property_value_pattern = false;
            return;
        }
        char[] charArray = s.toCharArray();
        int length = charArray.length;
        char[] array = new char[length];
        int i = 0;
    Label_0228:
        while (i < length) {
            switch (charArray[i]) {
                case ':': {
                    this._domain_length = i++;
                    break Label_0228;
                }
                case '=': {
                    i = ++i;
                    while (i < length && charArray[i++] != ':') {
                        if (i != length) {
                            continue;
                        }
                        throw new MalformedObjectNameException("Domain part must be specified");
                    }
                    continue;
                }
                case '\n': {
                    throw new MalformedObjectNameException("Invalid character '\\n' in domain name");
                }
                case '*':
                case '?': {
                    this._domain_pattern = true;
                    ++i;
                    continue;
                }
                default: {
                    ++i;
                    continue;
                }
            }
        }
        if (i == length) {
            throw new MalformedObjectNameException("Key properties cannot be empty");
        }
        System.arraycopy(charArray, 0, array, 0, this._domain_length);
        array[this._domain_length] = ':';
        int n = this._domain_length + 1;
        HashMap o = new HashMap();
        int n2 = 0;
        String[] array2 = new String[10];
        this._kp_array = new ObjectName.Property[10];
        this._property_list_pattern = false;
        this._property_value_pattern = false;
        while (i < length) {
            if (charArray[i] == '*') {
                if (this._property_list_pattern) {
                    throw new MalformedObjectNameException("Cannot have several '*' characters in pattern property list");
                }
                this._property_list_pattern = true;
                if (++i < length && charArray[i] != ',') {
                    throw new MalformedObjectNameException("Invalid character found after '*': end of name or ',' expected");
                }
                if (i == length) {
                    if (n2 == 0) {
                        this._kp_array = ObjectName._Empty_property_array;
                        this._ca_array = ObjectName._Empty_property_array;
                        this._propertyList = Collections.emptyMap();
                        break;
                    }
                    break;
                }
                else {
                    ++i;
                }
            }
            else {
                i = (i = i);
                if (charArray[i] == '=') {
                    throw new MalformedObjectNameException("Invalid key (empty)");
                }
                int c;
                while (i < length && (c = charArray[i++]) != '=') {
                    switch (c) {
                        case 10:
                        case 42:
                        case 44:
                        case 58:
                        case 63: {
                            throw new MalformedObjectNameException("Invalid character '" + ((c == '\n') ? "\\n" : ("" + (char)c)) + "' in key part of property");
                        }
                        default: {
                            continue;
                        }
                    }
                }
                if (charArray[i - 1] != '=') {
                    throw new MalformedObjectNameException("Unterminated key property part");
                }
                int n3 = i - i - 1;
                boolean n4 = false;
                int n5;
                int n6;
                if (i < length && charArray[i] == '\"') {
                    n5 = true;
                    int n7;
                    while (++i < length && (n7 = charArray[i]) != '\"') {
                        if (n7 == '\\') {
                            if (++i == length) {
                                throw new MalformedObjectNameException("Unterminated quoted value");
                            }
                            int c3;
                            switch (c3 = charArray[i]) {
                                case '\"':
                                case '*':
                                case '?':
                                case '\\':
                                case 'n': {
                                    continue;
                                }
                                default: {
                                    throw new MalformedObjectNameException("Invalid escape sequence '\\" + (char)c3 + "' in quoted value");
                                }
                            }
                        }
                        else {
                            if (n7 == '\n') {
                                throw new MalformedObjectNameException("Newline in quoted value");
                            }
                            switch (n7) {
                                case 42:
                                case 63: {
                                    n4 = true;
                                    continue;
                                }
                            }
                        }
                    }
                    if (i == length) {
                        throw new MalformedObjectNameException("Unterminated quoted value");
                    }
                    n6 = ++i - i;
                }
                else {
                    n5 = false;
                    int c2;
                    while (i < length && (c2 = charArray[i]) != ',') {
                        switch (c2) {
                            case 42:
                            case 63: {
                                n4 = true;
                                ++i;
                                continue;
                            }
                            case 10:
                            case 34:
                            case 58:
                            case 61: {
                                throw new MalformedObjectNameException("Invalid character '" + ((c2 == '\n') ? "\\n" : ("" + (char)c2)) + "' in value part of property");
                            }
                            default: {
                                ++i;
                                continue;
                            }
                        }
                    }
                    n6 = i - i;
                }
                if (i == length - 1) {
                    if (!(n5 == 0)) {
                        throw new MalformedObjectNameException("Invalid ending character `" + charArray[i] + "'");
                    }
                    throw new MalformedObjectNameException("Invalid ending comma");
                }
                else {
                    ++i;
                    Object o2;
                    if (n4 == 0) {
                        o2 = new Property(i, n3, n6);
                    }
                    else {
                        this._property_value_pattern = true;
                        o2 = new PatternProperty(i, n3, n6);
                    }
                    String substring = s.substring(i, i + n3);
                    if (n2 == array2.length) {
                        String[] array3 = new String[n2 + 10];
                        System.arraycopy(array2, 0, array3, 0, n2);
                        array2 = array3;
                    }
                    this.addProperty(o2, n2, o, array2[n2] = substring);
                    ++n2;
                }
            }
        }
        this.setCanonicalName(charArray, array, array2, o, n, n2);
        return;
    }
    
    private void construct(String s, Map<String, String> map) throws MalformedObjectNameException {
        if (s == null) {
            throw new NullPointerException("domain cannot be null");
        }
        if (map == null) {
            throw new NullPointerException("key property list cannot be null");
        }
        if (map.isEmpty()) {
            throw new MalformedObjectNameException("key property list cannot be empty");
        }
        if (!this.isDomain(s)) {
            throw new MalformedObjectNameException("Invalid domain: " + s);
        }
        StringBuilder sb = new StringBuilder();
        sb.append(s).append(':');
        this._domain_length = s.length();
        int size = map.size();
        this._kp_array = new ObjectName.Property[size];
        String[] array = new String[size];
        HashMap<String, ObjectName.Property> hashMap = new HashMap();
        int n = 0;
        for (Map.Entry entry : map.entrySet()) {
            if (sb.length() > 0) {
                sb.append(",");
            }
            String str = (String)entry.getKey();
            String str2;
            try {
                str2 = (String)entry.getValue();
            }
            catch (ClassCastException ex) {
                throw new MalformedObjectNameException(ex.getMessage());
            }
            int length = sb.length();
            checkKey(str);
            sb.append(str);
            array[n] = str;
            sb.append("=");
            int checkValue = checkValue(str2);
            sb.append(str2);
            Object o;
            if (checkValue == 0) {
                o = new ObjectName.Property(length, str.length(), str2.length());
            }
            else {
                this._property_value_pattern = true;
                o = new ObjectName.PatternProperty(length, str.length(), str2.length());
            }
            this.addProperty(o, n, hashMap, str);
            ++n;
        }
        int length2 = sb.length();
        char[] array2 = new char[length2];
        sb.getChars(0, length2, array2, 0);
        char[] array3 = new char[length2];
        System.arraycopy(array2, 0, array3, 0, this._domain_length + 1);
        this.setCanonicalName(array2, array3, array, hashMap, this._domain_length + 1, this._kp_array.length);
        return;
    }
    
    private void addProperty(ObjectName.Property property, int n, Map<String, ObjectName.Property> map, String str) throws MalformedObjectNameException {
        if (map.containsKey(str)) {
            throw new MalformedObjectNameException("key `" + str + "' already defined");
        }
        if (n == this._kp_array.length) {
            ObjectName.Property[] kp_array = new ObjectName.Property[n + 10];
            System.arraycopy(this._kp_array, 0, kp_array, 0, n);
            this._kp_array = kp_array;
        }
        map.put(str, this._kp_array[n] = property);
        return;
    }
    
    private void setCanonicalName(char[] array, char[] value, String[] array2, Map<String, ObjectName.Property> map, int n, int n2) {
        if (this._kp_array != ObjectName._Empty_property_array) {
            String[] a = new String[n2];
            ObjectName.Property[] kp_array = new ObjectName.Property[n2];
            System.arraycopy(array2, 0, a, 0, n2);
            Arrays.sort(a);
            array2 = a;
            System.arraycopy(this._kp_array, 0, kp_array, 0, n2);
            this._kp_array = kp_array;
            this._ca_array = new ObjectName.Property[n2];
            for (int i = 0; i < n2; ++i) {
                this._ca_array[i] = (ObjectName.Property)map.get(array2[i]);
            }
            for (int n3 = n2 - 1, j = 0; j <= n3; ++j) {
                ObjectName.Property property = this._ca_array[j];
                int n4 = property._key_length + property._value_length + 1;
                System.arraycopy(array, property._key_index, value, n, n4);
                property.setKeyIndex(n);
                n += n4;
                if (j != n3) {
                    value[n] = ',';
                    ++n;
                }
            }
        }
        if (this._property_list_pattern) {
            if (this._kp_array != ObjectName._Empty_property_array) {
                value[n++] = ',';
            }
            value[n++] = '*';
        }
        this._canonicalName = new String(value, 0, n).intern();
    }
    
    private static int parseKey(char[] array, int i) throws MalformedObjectNameException {
        i = (i = i);
        int length = array.length;
    Label_0172:
        while (i < length) {
            int c = array[i++];
            switch (c) {
                case 10:
                case 42:
                case 44:
                case 58:
                case 63: {
                    throw new MalformedObjectNameException("Invalid character in key: `" + ((c == '\n') ? "\\n" : ("" + (char)c)) + "'");
                }
                case 61: {
                    --i;
                    break Label_0172;
                }
                default: {
                    if (i < length) {
                        continue;
                    }
                    break Label_0172;
                }
            }
        }
        return i;
    }
    
    private static int[] parseValue(char[] array, int i) throws MalformedObjectNameException {
        boolean n = false;
        i = (i = i);
        int length = array.length;
        Label_0491: {
            if (array[i] == '\"') {
                if (++i == length) {
                    throw new MalformedObjectNameException("Invalid quote");
                }
                while (i < length) {
                    int c2 = array[i];
                    if (c2 == '\\') {
                        if (++i == length) {
                            throw new MalformedObjectNameException("Invalid unterminated quoted character sequence");
                        }
                        c2 = array[i];
                        switch (c2) {
                            case 42:
                            case 63:
                            case 92:
                            case 110: {
                                break;
                            }
                            case 34: {
                                if (i + 1 == length) {
                                    throw new MalformedObjectNameException("Missing termination quote");
                                }
                                break;
                            }
                            default: {
                                throw new MalformedObjectNameException("Invalid quoted character sequence '\\" + (char)c2 + "'");
                            }
                        }
                    }
                    else {
                        if (c2 == '\n') {
                            throw new MalformedObjectNameException("Newline in quoted value");
                        }
                        if (c2 == '\"') {
                            ++i;
                            break;
                        }
                        switch (c2) {
                            case 42:
                            case 63: {
                                n = true;
                                break;
                            }
                        }
                    }
                    if (++i >= length && c2 != '\"') {
                        throw new MalformedObjectNameException("Missing termination quote");
                    }
                    continue;
                }
                if ((i = i) < length && array[i++] != ',') {
                    throw new MalformedObjectNameException("Invalid quote");
                }
            }
            else {
                while (i < length) {
                    int c = array[i++];
                    switch (c) {
                        case 42:
                        case 63: {
                            n = true;
                            if (i < length) {
                                continue;
                            }
                            break Label_0491;
                        }
                        case 10:
                        case 58:
                        case 61: {
                            throw new MalformedObjectNameException("Invalid character `" + ((c == '\n') ? "\\n" : ("" + (char)c)) + "' in value");
                        }
                        case 44: {
                            --i;
                            break Label_0491;
                        }
                        default: {
                            if (i < length) {
                                continue;
                            }
                            break Label_0491;
                        }
                    }
                }
            }
        }
        return new int[] { i, n };
    }
    
    private static boolean checkValue(String s) throws MalformedObjectNameException {
        if (s == null) {
            throw new NullPointerException("Invalid value (null)");
        }
        int length = s.length();
        if (length == 0) {
            return false;
        }
        char[] charArray = s.toCharArray();
        int[] value = parseValue(charArray, 0);
        int n = value[0];
        int n2 = value[1] == 1;
        if (n < length) {
            throw new MalformedObjectNameException("Invalid character in value: `" + charArray[n] + "'");
        }
        return n2;
    }
    
    private static void checkKey(String s) throws MalformedObjectNameException {
        if (s == null) {
            throw new NullPointerException("Invalid key (null)");
        }
        int length = s.length();
        if (length == 0) {
            throw new MalformedObjectNameException("Invalid key (empty)");
        }
        char[] charArray = s.toCharArray();
        int key = parseKey(charArray, 0);
        if (key < length) {
            throw new MalformedObjectNameException("Invalid character in value: `" + charArray[key] + "'");
        }
        return;
    }
    
    private boolean isDomain(String s) {
        if (s == null) {
            return true;
        }
        int length = s.length();
        int i = 0;
        while (i < length) {
            switch (s.charAt(i++)) {
                case '\n':
                case ':': {
                    return false;
                }
                case '*':
                case '?': {
                    this._domain_pattern = true;
                    continue;
                }
            }
        }
        return true;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        String string;
        if (ObjectName.compat) {
            ObjectInputStream.GetField fields = objectInputStream.readFields();
            String s = (String)fields.get("propertyListString", "");
            if (fields.get("propertyPattern", false)) {
                s = ((s.length() == 0) ? "*" : (s + ",*"));
            }
            string = (String)fields.get("domain", "default") + ":" + s;
        }
        else {
            objectInputStream.defaultReadObject();
            string = (String)objectInputStream.readObject();
        }
        try {
            this.construct(string);
        }
        catch (NullPointerException ex) {
            throw new InvalidObjectException(ex.toString());
        }
        catch (MalformedObjectNameException ex2) {
            throw new InvalidObjectException(ex2.toString());
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (ObjectName.compat) {
            ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
            putFields.put("domain", this._canonicalName.substring(0, this._domain_length));
            putFields.put("propertyList", this.getKeyPropertyList());
            putFields.put("propertyListString", this.getKeyPropertyListString());
            putFields.put("canonicalName", this._canonicalName);
            putFields.put("pattern", this._domain_pattern || this._property_list_pattern);
            putFields.put("propertyPattern", this._property_list_pattern);
            objectOutputStream.writeFields();
        }
        else {
            objectOutputStream.defaultWriteObject();
            objectOutputStream.writeObject(this.getSerializedNameString());
        }
    }
    
    public static ObjectName getInstance(String s) throws MalformedObjectNameException, NullPointerException {
        return new ObjectName(s);
    }
    
    public static ObjectName getInstance(String s, String s2, String s3) throws MalformedObjectNameException {
        return new ObjectName(s, s2, s3);
    }
    
    public static ObjectName getInstance(String s, Hashtable<String, String> hashtable) throws MalformedObjectNameException {
        return new ObjectName(s, hashtable);
    }
    
    public static ObjectName getInstance(ObjectName objectName) {
        if (objectName.getClass().equals(ObjectName.class)) {
            return objectName;
        }
        return Util.newObjectName(objectName.getSerializedNameString());
    }
    
    public ObjectName(String s) throws MalformedObjectNameException {
        this._domain_length = 0;
        this._domain_pattern = false;
        this._property_list_pattern = false;
        this._property_value_pattern = false;
        this.construct(s);
    }
    
    public ObjectName(String s, String key, String value) throws MalformedObjectNameException {
        this._domain_length = 0;
        this._domain_pattern = false;
        this._property_list_pattern = false;
        this._property_value_pattern = false;
        this.construct(s, Collections.singletonMap(key, value));
    }
    
    public ObjectName(String s, Hashtable<String, String> hashtable) throws MalformedObjectNameException {
        this._domain_length = 0;
        this._domain_pattern = false;
        this._property_list_pattern = false;
        this._property_value_pattern = false;
        this.construct(s, hashtable);
    }
    
    public boolean isPattern() {
        return this._domain_pattern || this._property_list_pattern || this._property_value_pattern;
    }
    
    public boolean isDomainPattern() {
        return this._domain_pattern;
    }
    
    public boolean isPropertyPattern() {
        return this._property_list_pattern || this._property_value_pattern;
    }
    
    public boolean isPropertyListPattern() {
        return this._property_list_pattern;
    }
    
    public boolean isPropertyValuePattern() {
        return this._property_value_pattern;
    }
    
    public boolean isPropertyValuePattern(String anObject) {
        if (anObject == null) {
            throw new NullPointerException("key property can't be null");
        }
        int i = 0;
        while (i < this._ca_array.length) {
            ObjectName.Property property = this._ca_array[i];
            if (property.getKeyString(this._canonicalName).equals(anObject)) {
                return property instanceof ObjectName.PatternProperty;
            }
            ++i;
        }
        throw new IllegalArgumentException("key property not found");
    }
    
    public String getCanonicalName() {
        return this._canonicalName;
    }
    
    public String getDomain() {
        return this._canonicalName.substring(0, this._domain_length);
    }
    
    public String getKeyProperty(String s) {
        return (String)this._getKeyPropertyList().get(s);
    }
    
    private Map<String, String> _getKeyPropertyList() {
        synchronized (this) {
            if (this._propertyList == null) {
                this._propertyList = new HashMap();
                for (int i = this._ca_array.length - 1; i >= 0; --i) {
                    ObjectName.Property property = this._ca_array[i];
                    this._propertyList.put(property.getKeyString(this._canonicalName), property.getValueString(this._canonicalName));
                }
            }
        }
        return this._propertyList;
    }
    
    public Hashtable<String, String> getKeyPropertyList() {
        return new Hashtable(this._getKeyPropertyList());
    }
    
    public String getKeyPropertyListString() {
        if (this._kp_array.length == 0) {
            return "";
        }
        char[] value = new char[this._canonicalName.length() - this._domain_length - 1 - (this._property_list_pattern ? 2 : 0)];
        this.writeKeyPropertyListString(this._canonicalName.toCharArray(), value, 0);
        return new String(value);
    }
    
    private String getSerializedNameString() {
        char[] value = new char[this._canonicalName.length()];
        char[] charArray = this._canonicalName.toCharArray();
        int n = this._domain_length + 1;
        System.arraycopy(charArray, 0, value, 0, n);
        int writeKeyPropertyListString = this.writeKeyPropertyListString(charArray, value, n);
        if (this._property_list_pattern) {
            if (writeKeyPropertyListString == n) {
                value[writeKeyPropertyListString] = '*';
            }
            else {
                value[writeKeyPropertyListString] = ',';
                value[writeKeyPropertyListString + 1] = '*';
            }
        }
        return new String(value);
    }
    
    private int writeKeyPropertyListString(char[] array, char[] array2, int n) {
        if (this._kp_array.length == 0) {
            return n;
        }
        int length = this._kp_array.length;
        int n2 = length - 1;
        for (int i = 0; i < length; ++i) {
            ObjectName.Property property = this._kp_array[i];
            int n3 = property._key_length + property._value_length + 1;
            System.arraycopy(array, property._key_index, array2, n, n3);
            n += n3;
            if (i < n2) {
                array2[n++] = ',';
            }
        }
        return n;
    }
    
    public String getCanonicalKeyPropertyListString() {
        if (this._ca_array.length == 0) {
            return "";
        }
        int length = this._canonicalName.length();
        if (this._property_list_pattern) {
            length -= 2;
        }
        return this._canonicalName.substring(this._domain_length + 1, length);
    }
    
    @Override
    public String toString() {
        return this.getSerializedNameString();
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof ObjectName && this._canonicalName == ((ObjectName)o)._canonicalName);
    }
    
    @Override
    public int hashCode() {
        return this._canonicalName.hashCode();
    }
    
    public static String quote(String s) {
        StringBuilder sb = new StringBuilder("\"");
        for (int length = s.length(), i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            switch (char1) {
                case 10: {
                    char1 = 'n';
                    sb.append('\\');
                    break;
                }
                case 34:
                case 42:
                case 63:
                case 92: {
                    sb.append('\\');
                    break;
                }
            }
            sb.append((char)char1);
        }
        sb.append('\"');
        return sb.toString();
    }
    
    public static String unquote(String s) {
        StringBuilder sb = new StringBuilder();
        int length = s.length();
        if (length < 2 || s.charAt(0) != '\"' || s.charAt(length - 1) != '\"') {
            throw new IllegalArgumentException("Argument not quoted");
        }
        for (int i = 1; i < length - 1; ++i) {
            int c = s.charAt(i);
            if (c == '\\') {
                if (i == length - 2) {
                    throw new IllegalArgumentException("Trailing backslash");
                }
                c = s.charAt(++i);
                switch (c) {
                    case 'n': {
                        c = '\n';
                        break;
                    }
                    case '\"':
                    case '*':
                    case '?':
                    case '\\': {
                        break;
                    }
                    default: {
                        throw new IllegalArgumentException("Bad character '" + (char)c + "' after backslash");
                    }
                }
            }
            else {
                switch (c) {
                    case 10:
                    case 34:
                    case 42:
                    case 63: {
                        throw new IllegalArgumentException("Invalid unescaped character '" + (char)c + "' in the string to unquote");
                    }
                    default: {
                        break;
                    }
                }
            }
            sb.append((char)c);
        }
        return sb.toString();
    }
    
    @Override
    public boolean apply(ObjectName objectName) {
        if (objectName == null) {
            throw new NullPointerException();
        }
        if (objectName._domain_pattern || objectName._property_list_pattern || objectName._property_value_pattern) {
            return false;
        }
        if (!this._domain_pattern && !this._property_list_pattern && !this._property_value_pattern) {
            return this._canonicalName.equals(objectName._canonicalName);
        }
        return this.matchDomains(objectName) && this.matchKeys(objectName);
    }
    
    private final boolean matchDomains(ObjectName objectName) {
        if (this._domain_pattern) {
            return Util.wildmatch(objectName.getDomain(), this.getDomain());
        }
        return this.getDomain().equals(objectName.getDomain());
    }
    
    private final boolean matchKeys(ObjectName objectName) {
        if (this._property_value_pattern && !this._property_list_pattern && objectName._ca_array.length != this._ca_array.length) {
            return false;
        }
        if (this._property_value_pattern || this._property_list_pattern) {
            Map<String, String> getKeyPropertyList = objectName._getKeyPropertyList();
            ObjectName.Property[] ca_array = this._ca_array;
            String canonicalName = this._canonicalName;
            int i = ca_array.length - 1;
            while (i >= 0) {
                ObjectName.Property property = ca_array[i];
                String str = (String)getKeyPropertyList.get(property.getKeyString(canonicalName));
                if (str == null) {
                    return false;
                }
                if (this._property_value_pattern && property instanceof ObjectName.PatternProperty) {
                    if (!Util.wildmatch(str, property.getValueString(canonicalName))) {
                        return false;
                    }
                }
                else if (!str.equals(property.getValueString(canonicalName))) {
                    return false;
                }
                --i;
            }
            return true;
        }
        return objectName.getCanonicalKeyPropertyListString().equals(this.getCanonicalKeyPropertyListString());
    }
    
    @Override
    public void setMBeanServer(MBeanServer mBeanServer) {
    }
    
    @Override
    public int compareTo(ObjectName objectName) {
        if (objectName == this) {
            return 0;
        }
        int compareTo = this.getDomain().compareTo(objectName.getDomain());
        if (compareTo != 0) {
            return compareTo;
        }
        String keyProperty = this.getKeyProperty("type");
        String keyProperty2 = objectName.getKeyProperty("type");
        if (keyProperty == null) {
            keyProperty = "";
        }
        if (keyProperty2 == null) {
            keyProperty2 = "";
        }
        int compareTo2 = keyProperty.compareTo(keyProperty2);
        if (compareTo2 != 0) {
            return compareTo2;
        }
        return this.getCanonicalName().compareTo(objectName.getCanonicalName());
    }
    
    static {
        oldSerialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("domain", String.class), new ObjectStreamField("propertyList", Hashtable.class), new ObjectStreamField("propertyListString", String.class), new ObjectStreamField("canonicalName", String.class), new ObjectStreamField("pattern", Boolean.TYPE), new ObjectStreamField("propertyPattern", Boolean.TYPE) };
        newSerialPersistentFields = new ObjectStreamField[0];
        ObjectName.compat = false;
        try {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("jmx.serial.form"));
            ObjectName.compat = (s != null && s.equals("1.0"));
        }
        catch (Exception ex) {}
        if (ObjectName.compat) {
            serialPersistentFields = ObjectName.oldSerialPersistentFields;
            serialVersionUID = -5467795090068647408L;
        }
        else {
            serialPersistentFields = ObjectName.newSerialPersistentFields;
            serialVersionUID = 1081892073854801359L;
        }
        _Empty_property_array = new ObjectName.Property[0];
        WILDCARD = Util.newObjectName("*:*");
    }
    
    private static class Property
    {
        int _key_index;
        int _key_length;
        int _value_length;
        
        Property(int key_index, int key_length, int value_length) {
            this._key_index = key_index;
            this._key_length = key_length;
            this._value_length = value_length;
        }
        
        void setKeyIndex(int key_index) {
            this._key_index = key_index;
        }
        
        String getKeyString(String s) {
            return s.substring(this._key_index, this._key_index + this._key_length);
        }
        
        String getValueString(String s) {
            int beginIndex = this._key_index + this._key_length + 1;
            return s.substring(beginIndex, beginIndex + this._value_length);
        }
    }
    
    private static class PatternProperty extends ObjectName.Property
    {
        PatternProperty(int n, int n2, int n3) {
            super(n, n2, n3);
        }
    }
}
